#!/usr/bin/env ruby
#
#	iauephem: Read the contents of an IAU ephemerides table given via an URL
#			  and write it to a DAL data set.
#
#	CGI bin invocation syntax:
#		http://cfa-www.harvard.edu/ps/MPEph?O01=C/1999%20T1&d=2001%2001%2029
&u=m&i=10&l=200&e=-2
#
#	UL, Wed May  9 15:52:17 MET DST 2001
#
require 'Dal'
require 'STime'
require 'net/http'

include Math

class URL
	attr_reader(:hostname, :index)
	def initialize(url)
		m = /^http:\/\/([^\/]+)\/*(.*)/.match(url)
		@hostname = m ? m[1] : fail("Invalid URL #{url}")
		@index = m[2] && m[2].length>0 ? '/'+m[2] : '/index.html'
	end
end

class RightAscension
	attr_reader(:ra)
	def initialize(ra = 0.0)
		if ra.kind_of?(String)
			#	hh mm.fracmin
			m = /([+-]?)(\d+)\s+([\d\.]+)/.match(ra)
			@ra = 15.0 * m[2].to_f + 0.25*m[3].to_f
			@ra *= -1.0 if m[1][0]==?-
		end
		@ra *= Math::PI/180.0
	end
end

class Declination
	attr_reader(:dec)
	def initialize(dec = 0.0)
		if dec.kind_of?(String)
			#	deg mm.fracmin
			m = /([+-]?)(\d+)\s+([\d\.]+)/.match(dec)
			@dec = m[2].to_f + m[3].to_f/60.0
			@dec *= -1.0 if m[1][0]==?-
		end
		@dec *= Math::PI/180.0
	end
end

class EphemTable
	def initialize(setName, objectName, from = "unknown")
		@set = Dal::DataSet.new(setName, Dal::Create)
		@set.addAttribute("OBJECT").value = objectName
		@table = @set.addTable("TRACK")
		@table.addAttribute("OBJECT", "name of moving object").value =
							objectName
		@table.addComment("obtained from:")
		@table.addComment(from)
		@table.addComment("on " + Time.now().to_s)
		@table.addColumn("MJD", Dal::Real64, "d", "time")
		@table.addColumn("RA", Dal::Real32, "deg", "Right Ascension")
		@table.addColumn("DEC", Dal::Real32, "deg", "Declination")
		@table.addColumn("DELTA", Dal::Real32, "AU", "distance to earth")
		@table.addColumn("R", Dal::Real32, "AU", "distance to sun")
		@table.addColumn("ELONG", Dal::Real32, "deg", "angular distance from sun")
		@table.addColumn("PHASE", Dal::Real32, "deg", "angular earth/sun distance from object")
	end

	def appendRow(row)
		m = /^\s*(\d+\s+\d+\s+\d+)\s+(\d+\s+[\d\.]+)\s+([\+\-]?\d+\s+[\d\.]+)\s+([\d\.]+)\s+([\d\.]+)\s+([\d\.]+)\s+([\d\.]+).*$/.match(row)
		t = STime::STime.new(m[1].gsub(/ /, '-'))
		@table.appendRow([t.modifiedJulianDay,
						  RightAscension.new(m[2]).ra,
						  Declination.new(m[3]).dec],
						  m[4], m[5], m[6], m[7])
		t.release
	end

	def close
		@set.release
	end
end


if ARGV.length != 2
	puts("Usage: #{$0} <ephemerides-url> <datasetname>")
	exit(1)
end

url = URL.new(ARGV[0])
setName = ARGV[1]

puts("Hostname: #{url.hostname} - index: #{url.index}")

h = Net::HTTP.new(url.hostname)
resp, data = h.get(url.index, nil)
object = ""
if resp.message == "OK"
	arr = data.to_a
	arr.each_index { |i|
		m = /<title>(.*)<\/title>/.match(arr[i])
		if m
			object = m[1]
			object.gsub!(/--.*$/, '') if /--/.match(object)
			ephemTable = EphemTable.new(setName, object,
										url.hostname + url.index)
			arr[0..i] = nil
			arr.each_index { |i|
				if /^Date\s+TT\s+R\.\s*A\.\s*\(\d+\)\s+Decl/.match(arr[i])
					arr[0..i] = nil
					arr.each { |line|
						line.chomp!
						next if /^\s*$/.match(line)		# skip empty lines
						break if /^[^\d]/.match(line)	# end of table
						ephemTable.appendRow(line)
					}
				end
			}
			ephemTable.close
			break
		end
	}
end

puts("Object name: #{object}")
