#!/usr/bin/env ruby
#
#	iauephem: Read the contents of an IAU ephemerides table given via an URL
#			  and write it to a DAL data set.
#
#	UL, Wed May  9 15:52:17 MET DST 2001
#
require 'Dal'
require 'STime'

include Math

class RightAscension
	attr_reader(:ra)
	def initialize(ra = 0.0)
		if ra.kind_of?(String)
			#	hh mm.fracmin
			m = /([+-]?)(\d+)\s+([\d\.]+)/.match(ra)
			@ra = 15.0 * m[2].to_f + 0.25*m[3].to_f
			@ra *= -1.0 if m[1][0]==?-
		end
		@ra *= Math::PI/180.0
	end
end

class Declination
	attr_reader(:dec)
	def initialize(dec = 0.0)
		if dec.kind_of?(String)
			#	deg mm.fracmin
			m = /([+-]?)(\d+)\s+([\d\.]+)/.match(dec)
			@dec = m[2].to_f + m[3].to_f/60.0
			@dec *= -1.0 if m[1][0]==?-
		end
		@dec *= Math::PI/180.0
	end
end

class EphemTable
	def initialize(setName, objectName, from = "unknown")
		@set = Dal::DataSet.new(setName, Dal::Create)
		@set.addAttribute("OBJECT").value = objectName
		@table = @set.addTable("TRACK")
		@table.addAttribute("OBJECT", "name of moving object").value =
							objectName
		@table.addComment("obtained from:")
		@table.addComment(from)
		@table.addComment("on " + Time.now().to_s)
		@table.addColumn("MJD", Dal::Real64, "d", "time")
		@table.addColumn("RA", Dal::Real32, "deg", "Right Ascension")
		@table.addColumn("DEC", Dal::Real32, "deg", "Declination")
		@table.addColumn("DELTA", Dal::Real32, "AU", "distance to earth")
		@table.addColumn("R", Dal::Real32, "AU", "distance to sun")
		@table.addColumn("ELONG", Dal::Real32, "deg", "angular distance from sun")
		@table.addColumn("PHASE", Dal::Real32, "deg", "angular earth/sun distance from object")
	end

	def appendRow(row)
		m = /^\s*(\d+\s+\d+\s+\d+\s+\d+)\s+(\d+\s+[\d\.]+)\s+([\+\-]?\d+\s+[\d\.]+)\s+([\d\.]+)\s+([\d\.]+)\s+([\d\.]+)\s+([\d\.]+).*$/.match(row)
		mm = /(\d+) (\d+) (\d+) (\d\d)(\d\d)/.match(m[1])
		str = mm[1]+'-'+mm[2]+'-'+mm[3]+'T'+mm[4]+':'+mm[5]+':00'
		t = STime::STime.new(mm[1]+'-'+mm[2]+'-'+mm[3]+'T'+mm[4]+':'+mm[5]+':00')
#		puts("time=#{str}")
#		puts("ra=#{m[2]}")
#		puts("dec=#{m[3]}")
#		puts("delta=#{m[4]}")
#		puts("r=#{m[5]}")
#		puts("elong=#{m[6]}")
#		puts("phase=#{m[7]}")
		@table.appendRow([t.modifiedJulianDay,
						  RightAscension.new(m[2]).ra*180.0/Math::PI,
						  Declination.new(m[3]).dec*180.0/Math::PI,
						  m[4].to_f, m[5].to_f, m[6].to_f, m[7].to_f])
		t.release
	end

	def close
		@set.release
	end
end


if ARGV.length != 2
	puts("Usage: #{$0} <ephemerides-file> <datasetname>")
	exit(1)
end

file = File.new(ARGV[0], "r")
setName = ARGV[1]

object = ""
arr = file.readlines
arr.each_index { |i|
	m = /<title>(.*)<\/title>/.match(arr[i])
	if m
		object = m[1]
		object.gsub!(/--.*$/, '') if /--/.match(object)
		ephemTable = EphemTable.new(setName, object, ARGV[0])
		arr[0..i] = nil
		arr.each_index { |i|
			if /^Date\s+UT\s+R\.\s*A\.\s*\(J?\d+\)\s+Decl/.match(arr[i])
				arr[0..i] = nil
				arr.each { |line|
					line.chomp!
					next if /^\s+/.match(line)		# skip empty lines
					break if /^[^\d]/.match(line)	# end of table
					ephemTable.appendRow(line)
				}
			end
		}
		ephemTable.close
		break
	end
}

#puts("Object name: #{object}")

