//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//						(c) 1997 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	movecalc.cc --- CALCulate image coordinates for MOVing object
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		UL, 2001-05-11: 1.0.0: - inital useful version
//
//	AUTHOR:
//		U. Lammers, Fri May 11 15:01:42 MET DST 2001
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /home/sasbuild/repositories/sasdev/task_devel/packages/movecalc/movecalc/src/movecalc.cc,v 1.3 2002/02/28 12:41:06 ulammers Exp $
//
#include <cmath>

#include "param.h"
#include "Dal.h"
#include "errstr.h"
#include "Tracer.h"

#include "slatec++.h"
#include "CalOalUtility.h"
#include "PhysicalConstants.h"
#include "OalServer.h"
#include "Celestial.h"
#include "Xmm.h"

#include "movecalc.h"

static int32 nearestInt(double x)
{
	return (int32)(x<0. ? std::ceil(x-0.5) : std::floor(x+.5));
}

void movecalc()
{
	PROC

	std::string tabName = stringParameter("table"),
			    trackFile = stringParameter("objtrack"),
			    timecolName = stringParameter("timecol"),
			    xcolName = stringParameter("xcol"),
			    ycolName = stringParameter("ycol"),
			    movxcolName = stringParameter("xmovcol"),
			    movycolName = stringParameter("ymovcol");

	DataSet *set = dataSetServer->open(tabName, Dal::Modify);
	Table *table = set->table(tabName);

	Column *tcol = table->column(timecolName),
		   *xcol = table->column(xcolName),
		   *ycol = table->column(ycolName),
		   *xmovcol = table->addColumn(movxcolName, Dal::Int32),
		   *ymovcol = table->addColumn(movycolName, Dal::Int32);

	ColumnData *tcold = tcol->data(),
			   *xcold = xcol->data(),
			   *ycold = ycol->data(),
			   *xmovcold = xmovcol->data(),
			   *ymovcold = ymovcol->data();

	int32 *x = xcold->int32Data(),
		  *y = ycold->int32Data(),
		  *xmov = xmovcold->int32Data(),
	      *ymov = ymovcold->int32Data();
	real64 *t = tcold->real64Data();

	//	fixed image reference point
	EquatorialDirection refPoint(RightAscension(xcol->attribute("TCRVL")
									->asReal() * DEG_TO_RAD),
								 Declination(ycol->attribute("TCRVL")
									->asReal() * DEG_TO_RAD));

	double refX = xcol->attribute("TCRPX")->asReal(),
		   refY = ycol->attribute("TCRPX")->asReal(),
		   deltaX = xcol->attribute("TCDLT")->asReal() * DEG_TO_RAD,
		   deltaY = ycol->attribute("TCDLT")->asReal() * DEG_TO_RAD;
	WCS::TangentialGnomonicProjection wcsfix(refPoint,
										refX, deltaX, refY, deltaY);

	//
	//	read object track
	//
	DataSet *objTrackSet = dataSetServer->open(trackFile, Dal::Read);
	Table *trackTab = objTrackSet->table(0);
	std::vector<double> mjdvec, ravec, decvec, deltavec;
	CalOalUtility::fillVector(trackTab, "MJD", mjdvec);
	CalOalUtility::fillVector(trackTab, "RA", ravec);
	CalOalUtility::fillVector(trackTab, "DEC", decvec);
	CalOalUtility::fillVector(trackTab, "DELTA", deltavec);	//	[AU]
	dataSetServer->close(objTrackSet);

	Slatec::CubicHermiteInterpolant<double> objra(mjdvec, ravec),
											objdec(mjdvec, decvec),
											delta(mjdvec, deltavec);

	//
	//	obtain PositionLocator
	//
	const PositionLocator *xmmPosLoc = oalServer.positionLocator();

	//
	//	event loop
	//
	unsigned N = table->rows();
	for (unsigned n=0; n<N; ++n) {
		double mjd = (Xmm::missionReferenceTime
					  + t[n]).modifiedJulianDay().full();

		RightAscension	instRa(objra(mjd)*DEG_TO_RAD);
		Declination		instDec(objdec(mjd)*DEG_TO_RAD);
		SAS_DEBUG(cout<<"MJD="<<setprecision(8)<<mjd
					<<" R.A.="<<instRa.angle()*RAD_TO_DEG
					<<" Dec="<<instDec.angle()*RAD_TO_DEG<<endl;)
		EquatorialDirection instRefPoint(instRa, instDec);
		//	geocentric parallax shift
		instRefPoint.toApparentDirection(
						xmmPosLoc->stateVector(t[n]).position,
						delta(mjd) * PhysicalConstants::AU / 1000.0);

		WCS::TangentialGnomonicProjection wcsmov(instRefPoint,
											0., 10.*deltaX, 0., 10.*deltaY);

		WCS::ZenithalProjection::NativePlaneCoordinates pfix(x[n], y[n]),
													    pmov;

		pmov = wcsmov.toNativeSystem(wcsfix.toStandardSystem(pfix));
		xmov[n] = nearestInt(pmov.first);
		ymov[n] = nearestInt(pmov.second);
	}

	delete xcold;
	delete ycold;
	delete tcold;
	delete xmovcold;
	delete ymovcold;
	dataSetServer->close(set);

	ENDPROC
} // movecalc()
